import os
import tarfile
from typing import List, NamedTuple

import requests
import yaml
from hapi.chart.chart_pb2 import Chart
from pyhelm.chartbuilder import ChartBuilder
from pyhelm.tiller import Tiller

from . import exceptions
from .port_forwarding import PortForwarder
from ..logging import get_logger


DEFAULT_REPO_FILE = "~/.helm/repository/repositories.yaml"


class _ChartKinds(NamedTuple):
    directory: str = "directory"
    repo: str = "repo"
    git: str = "git"


ChartKinds = _ChartKinds()


class Repo:
    def __init__(self, **kwargs):
        self.ca_file = kwargs.get("caFile")
        self.cache = kwargs.get("cache")
        self.cert_file = kwargs.get("certFile")
        self.key_file = kwargs.get("keyFile")
        self.name = kwargs.get("name")
        self.password = kwargs.get("password")
        self.url = kwargs.get("url")
        self.username = kwargs.get("username")

    def __repr__(self) -> str:
        return f"<deploy_py.helm.Repo: {self.name}>"


class Client:
    def __init__(self, repo_file: str = DEFAULT_REPO_FILE, verbosity: int = 0):
        self.logger = get_logger("deploy.helm.logger", level=verbosity)
        with open(os.path.expanduser(repo_file)) as repo_file:
            repos = yaml.load(repo_file, Loader=yaml.SafeLoader)
        self.repos = {repo["name"]: Repo(**repo) for repo in repos["repositories"]}

    @staticmethod
    def exclude_function(tarinfo):
        if ".helmignore" in tarinfo.name:
            return None
        else:
            return tarinfo

    def make_tarfile(self, output_filename: str, source_dir: str):
        with tarfile.open(output_filename, "w:gz") as tar:
            tar.add(source_dir, arcname=os.path.basename(source_dir), filter=self.exclude_function)

    def get_repo(self, repo_name: str) -> Repo:
        return self.repos[repo_name]

    def delete_chart(self, repo_name: str, chart_name: str, version: str) -> requests.Response:
        repo = self.get_repo(repo_name)
        url = f"{repo.url}/{chart_name}/{version}"
        self.logger.info(f"Removing previous Helm chart {chart_name}/{version}")
        resp = requests.delete(url, auth=(repo.username, repo.password))
        return resp

    def package(self, name: str, version: str, src_dir: str, out_dir: str) -> str:
        self.logger.info(f"Creating Helm chart {name}:{version}...")
        out_name = os.path.join(os.path.expanduser(out_dir), f"{name}-{version}.tgz")
        self.make_tarfile(out_name, src_dir)
        return out_name

    def upload(self, repo_name: str, file_path: str):
        with open(file_path, "rb") as chart:
            files = {"chart": chart}
            repo = self.get_repo(repo_name)
            self.logger.info(f"Uploading Helm chart to {repo.url}/{chart.name}")
            resp = requests.post(f"{repo.url}/api/charts", files=files, auth=(repo.username, repo.password))
            if resp.status_code not in range(200, 299):
                raise exceptions.HelmUploadError(f"error uploading Helm chart: {resp.json()}")

    @staticmethod
    def list() -> List[Chart]:
        with PortForwarder("tiller-deploy-6469cb697f-fc74x", "sre-test") as port:
            t = Tiller("localhost", port=port)
            releases = t.list_charts()
            return releases

    @staticmethod
    def build_chart(name: str, version: str, kind: str, location: str):
        chart_options = {
            "name": name,
            "version": version,
            "source": {
                "type": kind,
                "location": location
            }
        }
        return ChartBuilder(chart_options)

    def update_from_directory(self, name: str, version: str, chart_path: str):
        chart = self.build_chart(name, version, ChartKinds.directory, chart_path)
        with PortForwarder("tiller-deploy-6469cb697f-fc74x", "sre-test") as port:
            t = Tiller("localhost", port=port)
            try:
                resp = t.update_release(
                    chart.get_helm_chart(),
                    dry_run=False,
                    namespace='sre',
                    name=name,
                    install=True
                )
                if resp.release.info.status.code == resp.release.info.status.FAILED:
                    self.logger.error(f"release failed: {resp.release.info.description}")
                else:
                    self.logger.info(f"release status is {resp.release.info}")
            except Exception as err:
                import ipdb; ipdb.set_trace()
                print(123)


    def update_index(self, chart_dir: str, repo_name: str):
        repo = self.get_repo(repo_name)
        self.logger.info("WIP: Updating Helm index file...")
        self.logger.warn(f"updating index not yet supported; run this to update:\n\nhelm repo index {chart_dir} - -url {repo.url}")
        self.logger.info("WIP: Updating Helm chart repos...")
        self.logger.warn(f"updating repos yet supported; run this to update:\n\nhelm repo update")
